package com.dylanvann.fastimage;

import android.app.Activity;
import android.graphics.drawable.Drawable;
import android.util.Base64;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.Key;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.load.engine.cache.DiskCache;
import com.bumptech.glide.load.model.GlideUrl;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URI;

class FastImageViewModule extends ReactContextBaseJavaModule {

    private static final String REACT_CLASS = "FastImageView";
    private static int counter = 0;

    FastImageViewModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }

    @NonNull
    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @ReactMethod
    public void preload(final ReadableArray sources, final Promise promise) {
        final Activity activity = getCurrentActivity();
        if (activity == null) return;
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                final int total = sources.size();
                counter = sources.size();
                for (int i = 0; i < sources.size(); i++) {
                    final ReadableMap source = sources.getMap(i);
                    final FastImageSource imageSource = FastImageViewConverter.getImageSource(activity, source);

                    Glide
                            .with(activity.getApplicationContext())
                            // This will make this work for remote and local images. e.g.
                            //    - file:///
                            //    - content://
                            //    - res:/
                            //    - android.resource://
                            //    - data:image/png;base64
                            .load(
                                    imageSource.isBase64Resource() ? imageSource.getSource() :
                                            imageSource.isResource() ? imageSource.getUri() : imageSource.getGlideUrl()
                            )
                            .apply(FastImageViewConverter.getOptions(activity, imageSource, source))
                            .addListener(new RequestListener<Drawable>() {
                                @Override
                                public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Drawable> target, boolean isFirstResource) {
                                    counter--;
                                    if (counter == 0) {
                                        promise.resolve(total);
                                    }
                                    return false;
                                }

                                @Override
                                public boolean onResourceReady(Drawable resource, Object model, Target<Drawable> target, DataSource dataSource, boolean isFirstResource) {
                                    counter--;
                                    if (counter == 0) {
                                        promise.resolve(total);
                                    }
                                    return false;
                                }
                            })
                            .preload();
                }
            }
        });
    }

    @ReactMethod
    public void preloadSingle(final ReadableMap source, final Promise promise) {
        final Activity activity = getCurrentActivity();
        if (activity == null) return;
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                final FastImageSource imageSource = FastImageViewConverter.getImageSource(activity, source);
                Glide
                        .with(activity.getApplicationContext())
                        .load(
                                imageSource.isBase64Resource() ? imageSource.getSource() :
                                        imageSource.isResource() ? imageSource.getUri() : imageSource.getGlideUrl()
                        )
                        .apply(FastImageViewConverter.getOptions(activity, imageSource, source))
                        .addListener(new RequestListener<Drawable>() {
                            @Override
                            public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Drawable> target, boolean isFirstResource) {
                                promise.reject("error", e);
                                return false;
                            }

                            @Override
                            public boolean onResourceReady(Drawable resource, Object model, Target<Drawable> target, DataSource dataSource, boolean isFirstResource) {
                                File cachedFile = FastImageDiskCacheFactory.getDiskCache().get(imageSource.getGlideUrl());
                                if (cachedFile != null && cachedFile.exists()) {
                                    promise.resolve(cachedFile.getAbsolutePath());
                                } else {
                                    promise.resolve(null);
                                }
                                return false;
                            }
                        })
                        .preload();
            }
        });
    }

    @ReactMethod
    public void putSource(String url, final String body, Promise promise) {

        Key key = new GlideUrl(url);
        FastImageDiskCacheFactory.getDiskCache().put(key, new DiskCache.Writer() {
            @Override
            public boolean write(@NonNull File file) {

                byte[] data = Base64.decode(body, Base64.DEFAULT);
                try {
                    OutputStream stream = new FileOutputStream(file);
                    stream.write(data);
                    stream.flush();
                    stream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    return false;
                }
                return true;
            }
        });
        File cachedFile = FastImageDiskCacheFactory.getDiskCache().get(key);
        if (cachedFile != null && cachedFile.exists()) {
            promise.resolve(cachedFile.getAbsolutePath());
        } else {
            promise.reject("error", "file not found");
        }
    }

    @ReactMethod
    public void putFile(String url, final String path, Promise promise) {

        Key key = new GlideUrl(url);
        FastImageDiskCacheFactory.getDiskCache().put(key, new DiskCache.Writer() {
            @Override
            public boolean write(@NonNull File file) {
                try {
                    FileInputStream inputStream;
                    if (path.startsWith("file://")) {
                        inputStream = new FileInputStream(URI.create(path).getPath());
                    } else {
                        inputStream = new FileInputStream(path);
                    }
                    byte [] b = new byte[1024];
                    OutputStream outputStream = new FileOutputStream(file);

                    while(true){
                        int temp = inputStream.read(b, 0, b.length);
                        //如果temp = -1的时候，说明读取完毕
                        if(temp == -1){
                            break;
                        }
                        outputStream.write(b, 0, temp);
                    }
                    outputStream.flush();
                    outputStream.close();
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    return false;
                }
                return true;
            }
        });
        File cachedFile = FastImageDiskCacheFactory.getDiskCache().get(key);
        if (cachedFile != null && cachedFile.exists()) {
            promise.resolve(cachedFile.getAbsolutePath());
        } else {
            promise.reject("error", "file not found");
        }
    }

    @ReactMethod
    public void cacheExists(String url, final String body, Promise promise) {
        Key key = new GlideUrl(url);
        File cachedFile = FastImageDiskCacheFactory.getDiskCache().get(key);
        if (cachedFile != null && cachedFile.exists()) {
            promise.resolve(cachedFile.getAbsolutePath());
        } else {
            promise.resolve(null);
        }
    }
}
